from collections.abc import Iterable, Iterator

from galactic.algebras.closure import AbstractClosure, ClosedEnumerableMixin
from galactic.algebras.lattice import Element
from galactic.algebras.set import FrozenFIFOSet

class Multiple(ClosedEnumerableMixin[int], Element):  # type: ignore[misc]
    def __init__(
        self,
        connection: IntegerConnection,
        elements: Iterable[int] | None = None,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    def __contains__(self, value: object) -> bool: ...
    @property
    def closure(self) -> AbstractClosure[Multiple, int]: ...

class Divisors(ClosedEnumerableMixin[int], Element):  # type: ignore[misc]
    def __init__(
        self,
        connection: IntegerConnection,
        elements: Iterable[int] | None = None,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    def __contains__(self, value: object) -> bool: ...
    @property
    def closure(self) -> AbstractClosure[Divisors, int]: ...

class MultiplePolarity:
    def __init__(self, connection: IntegerConnection) -> None: ...
    def __call__(
        self,
        *others: Divisors,
        elements: Iterable[int] | None = None,
    ) -> Multiple: ...

class DivisorPolarity:
    def __init__(self, connection: IntegerConnection) -> None: ...
    def __call__(
        self,
        *others: Multiple,
        elements: Iterable[int] | None = None,
    ) -> Divisors: ...

class IntegerConnection:
    def __init__(self, domain: Iterable[int], co_domain: Iterable[int]) -> None: ...
    @property
    def domain(self) -> FrozenFIFOSet[int]: ...
    @property
    def co_domain(self) -> FrozenFIFOSet[int]: ...
    @property
    def polarities(self) -> tuple[
        MultiplePolarity,
        DivisorPolarity,
    ]: ...
    @property
    def closures(self) -> tuple[
        AbstractClosure[Divisors, int],
        AbstractClosure[Multiple, int],
    ]: ...
